<?php

namespace Drupal\static_export\Exporter\Output\Config;

/**
 * An interface for factories that create ExporterOutputConfigInterface objects.
 */
interface ExporterOutputConfigFactoryInterface {

  /**
   * Get export base directory.
   *
   * @return string
   *   The export base directory. This is a fixed value that is defined in this
   *   module's services.yml.
   */
  public function getBaseDir(): string;

  /**
   * Creates a ExporterOutputConfigInterface object.
   *
   * Instead of having a single parameter called $path, this method is split
   * out into its different parts ($dir, $filename, $extension). This avoids
   * having to deal with string concatenations, having to parse strings to get
   * the path's extension (required to get its format), makes it simpler to
   * override this configuration when an event is dispatched by a exporter, and
   * simplifies the addition of dynamically calculated subdirectories to the
   * path (e.g.- EntityExporterInterface::OPTIONAL_SUB_DIR_TOKEN).
   *
   * The only downside of this approach is that, since we want to maintain the
   * natural order of elements ($dir/$filename.$extension) we don't allow a
   * NULL value for $dir. The reason is that defining an optional parameter
   * before a required one ($filename) is bad practice.
   *
   * Here there are some example of how should this method be used:
   *
   * @code
   *  $factory->create('my-dir/my-sub-dir', 'my-file', 'json')
   *  $factory->create('my-dir', 'my-file', 'json')
   *  $factory->create('', 'my-file', 'json')
   *  $factory->create('', 'my-file')
   * @endcode
   *
   * If output format or language need to be set, it can be done as follows:
   * @code
   *  $factory->create('my-dir', 'my-file', 'json')
   *  ->getDefinition()
   *  ->setFormat('json')
   *  ->setLanguage($language);
   * @endcode
   *
   * @param string $dir
   *   The export directory, relative to base dir inside data dir. It can
   *   contain subdirectories.
   *   Optional, it can be an empty string. It cannot be null, because we want
   *   to maintain the natural order of elements in a path
   *   ($dir/$filename.$extension) and defining an optional parameter before a
   *   required one ($filename) is bad practice.
   *   Hence, if an empty string is passed, we consider it to be null.
   * @param string $filename
   *   Export filename.
   *   Required. It must be a string with letters or numbers, with a minimum
   *   length of one character. Not meeting this requirement throws an error.
   * @param string|null $extension
   *   Export extension.
   *   Optional, it can be null or an empty string.
   *
   * @return \Drupal\static_export\Exporter\Output\Config\ExporterOutputConfigInterface
   *   A exporter output config.
   */
  public function create(string $dir, string $filename, string $extension = NULL): ExporterOutputConfigInterface;

}
