<?php

namespace Drupal\static_export\Plugin\static_export\Exporter\Locale;

use Drupal\static_export\Exporter\Output\Config\ExporterOutputConfigInterface;
use Drupal\static_export\Exporter\Type\Locale\LocaleExporterPluginBase;
use Drupal\static_suite\StaticSuiteUserException;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Exporter for localized strings.
 *
 * @StaticLocaleExporter(
 *  id = "default_locale",
 *  label = @Translation("Localized string exporter"),
 *  description = @Translation("Exports localized strings to filesystem. Default locale exporter provided by Static Suite."),
 * )
 */
class LocaleExporter extends LocaleExporterPluginBase {

  protected const PARAM_LANG = "language";

  /**
   * Locale storage.
   *
   * @var \Drupal\locale\StringStorageInterface
   */
  protected $localeStorage;

  /**
   * {@inheritdoc}
   */
  protected function setExtraDependencies(ContainerInterface $container): void {
    $this->localeStorage = $container->get("locale.storage");
  }

  /**
   * {@inheritdoc}
   *
   * @return \Drupal\locale\StringInterface[]
   *   Array of StringInterface
   */
  public function getExporterItem() {
    $langCode = $this->options[self::PARAM_LANG];
    if (empty($langCode)) {
      return NULL;
    }

    if (empty($this->exporterItem[$langCode])) {
      $this->exporterItem[$langCode] = $this->localeStorage->getTranslations([
        'language' => $langCode,
        'translated' => TRUE,
      ]);
    }

    return $this->exporterItem[$langCode];
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemId() {
    return 'localized_strings';
  }

  /**
   * {@inheritdoc}
   */
  public function getExporterItemLabel() {
    return 'Localized strings ' . $this->options[self::PARAM_LANG];
  }

  /**
   * {@inheritdoc}
   */
  public function checkParams(array $options): bool {
    $langCode = $options[self::PARAM_LANG];
    if (!isset($langCode)) {
      throw new StaticSuiteUserException("Param '" . self::PARAM_LANG . "' is not defined.");
    }

    $enabledLanguages = $this->languageManager->getLanguages();
    if (empty($enabledLanguages[$langCode])) {
      throw new StaticSuiteUserException("Language '" . $langCode . "' is not enabled on this site.");
    }

    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  protected function getOutputDefinition(): ExporterOutputConfigInterface {
    $config = $this->configFactory->get('static_export.settings');
    $filename = 'locale.' . $this->options[self::PARAM_LANG];
    $format = $config->get('exportable_locale.format');

    // Load the OutputFormatter plugin definition to get its extension.
    $definitions = $this->outputFormatterManager->getDefinitions();
    $extension = !empty($definitions[$format]) ? $definitions[$format]['extension'] : $format;

    $outputConfig = $this->exporterOutputConfigFactory->create('', $filename, $extension);
    $outputConfig->getDefinition()->setFormat($format);
    $language = $this->languageManager->getLanguage($this->options[self::PARAM_LANG]);
    if ($language) {
      $outputConfig->getDefinition()->setLanguage($language);
    }
    return $outputConfig;
  }

  /**
   * {@inheritdoc}
   */
  protected function calculateDataFromResolver() {
    $localeData = [];
    foreach ($this->getExporterItem() as $locale) {
      $localeData[$locale->source] = $locale->getString();
    }

    return $localeData;
  }

}
